// -*- C++ -*-
/*!
* @file  RTC_KinectOpenNI2.cpp
* @brief ModuleDescription
* @date $Date$
*
* $Id$
*/

#include "RTC_KinectOpenNI2.h"

// Module specification
// <rtc-template block="module_spec">
static const char* rtc_kinectopenni2_spec[] =
{
	"implementation_id", "RTC_KinectOpenNI2",
	"type_name",         "RTC_KinectOpenNI2",
	"description",       "ModuleDescription",
	"version",           "1.0.0",
	"vendor",            "AIS",
	"category",          "Category",
	"activity_type",     "PERIODIC",
	"kind",              "DataFlowComponent",
	"max_instance",      "1",
	"language",          "C++",
	"lang_type",         "compile",
	// Configuration variables
	"conf.default.alignmentFlag", "0",
	// Widget
	"conf.__widget__.alignmentFlag", "text",
	// Constraints
	"conf.__constraints__.alignmentFlag", "(0,1)",
	""
};
// </rtc-template>

/*!
* @brief constructor
* @param manager Maneger Object
*/
RTC_KinectOpenNI2::RTC_KinectOpenNI2(RTC::Manager* manager)
	// <rtc-template block="initializer">
	: RTC::DataFlowComponentBase(manager),
	m_rgbdataOut("rgbdata", m_rgbdata),
	m_skeltondataOut("skeltondata", m_skeltondata)

	// </rtc-template>
{
}

/*!
* @brief destructor
*/
RTC_KinectOpenNI2::~RTC_KinectOpenNI2()
{
}



RTC::ReturnCode_t RTC_KinectOpenNI2::onInitialize()
{
	// Registration: InPort/OutPort/Service
	// <rtc-template block="registration">
	// Set InPort buffers

	// Set OutPort buffer
	addOutPort("rgbdata", m_rgbdataOut);
	addOutPort("skeltondata", m_skeltondataOut);

	// Set service provider to Ports

	// Set service consumers to Ports

	// Set CORBA Service Ports

	// </rtc-template>

	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("alignmentFlag", m_alignmentFlag, "0");
	// </rtc-template>
	kinect.Initialize();
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_KinectOpenNI2::onFinalize()
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_KinectOpenNI2::onStartup(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_KinectOpenNI2::onShutdown(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RTC_KinectOpenNI2::onActivated(RTC::UniqueId ec_id)
{
	kinect.Colorimagestart();
	kinect.Depthimagestart();
	/*if(m_alignmentFlag == 0){
		kinect.Doalignment();
	}*/

	return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_KinectOpenNI2::onDeactivated(RTC::UniqueId ec_id)
{
	//kinect.ClearStream();
	cv::destroyAllWindows();
	return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_KinectOpenNI2::onExecute(RTC::UniqueId ec_id)
{
	int changeIndex = 999;
	kinect.WaitforStream(changeIndex);
		
	if(changeIndex == 0){
		kinect.Getcolorimage(colorimage);
		kinect.getskeltondata(skeltonpoints);
		
		m_rgbdata.width = colorimage.size().width;
		m_rgbdata.height = colorimage.size().height;
		m_rgbdata.pixels.length(colorimage.size().width * colorimage.size().height * 3);
		memcpy((void*)(&m_rgbdata.pixels[0]), (void*)(&colorimage.data[0]), m_rgbdata.pixels.length());
		m_rgbdataOut.write();
	
		m_skeltondata.data.length(skeltonpoints.size() * 3);
		for(int i=0; i<skeltonpoints.size(); i++){
			m_skeltondata.data[3*i+0] = (short)skeltonpoints[i].x;
			m_skeltondata.data[3*i+1] = (short)skeltonpoints[i].y;
			m_skeltondata.data[3*i+2] = (short)skeltonpoints[i].z;
		}
		m_skeltondataOut.write();
		int key = cv::waitKey(10);
	}
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_KinectOpenNI2::onAborting(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_KinectOpenNI2::onError(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_KinectOpenNI2::onReset(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_KinectOpenNI2::onStateUpdate(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_KinectOpenNI2::onRateChanged(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/



extern "C"
{

	void RTC_KinectOpenNI2Init(RTC::Manager* manager)
	{
		coil::Properties profile(rtc_kinectopenni2_spec);
		manager->registerFactory(profile,
			RTC::Create<RTC_KinectOpenNI2>,
			RTC::Delete<RTC_KinectOpenNI2>);
	}

};


